@@ -7,7 +7,8 @@
 #define nsHTMLDocument_h___
 
 #include "mozilla/Attributes.h"
-#include "nsDocument.h"
+#include "nsContentList.h"
+#include "mozilla/dom/Document.h"
 #include "nsIHTMLDocument.h"
 #include "nsIHTMLCollection.h"
 #include "nsIScriptElement.h"
@@ -15,39 +16,48 @@
 
 #include "PLDHashTable.h"
 #include "nsIHttpChannel.h"
-#include "nsHTMLStyleSheet.h"
 #include "nsThreadUtils.h"
-#include "nsICommandManager.h"
 #include "mozilla/dom/HTMLSharedElement.h"
 #include "mozilla/dom/BindingDeclarations.h"
 
+class nsCommandManager;
 class nsIURI;
 class nsIDocShell;
 class nsICachingChannel;
-class nsIWyciwygChannel;
 class nsILoadGroup;
 
 namespace mozilla {
 namespace dom {
 class HTMLAllCollection;
+template <typename T>
+struct Nullable;
+class WindowProxyHolder;
 }  // namespace dom
 }  // namespace mozilla
 
-class nsHTMLDocument : public nsDocument, public nsIHTMLDocument {
+class nsHTMLDocument : public mozilla::dom::Document, public nsIHTMLDocument {
+ protected:
+  typedef mozilla::net::ReferrerPolicy ReferrerPolicy;
+  typedef mozilla::dom::Document Document;
+  typedef mozilla::Encoding Encoding;
+  template <typename T>
+  using NotNull = mozilla::NotNull<T>;
+
  public:
-  using nsDocument::GetPlugins;
-  using nsDocument::SetDocumentURI;
+  using Document::GetPlugins;
+  using Document::SetDocumentURI;
 
   nsHTMLDocument();
   virtual nsresult Init() override;
 
   NS_DECL_ISUPPORTS_INHERITED
-  NS_DECL_CYCLE_COLLECTION_CLASS_INHERITED(nsHTMLDocument, nsDocument)
+  NS_DECL_CYCLE_COLLECTION_CLASS_INHERITED(nsHTMLDocument, Document)
 
-  // nsIDocument
+  // Document
   virtual void Reset(nsIChannel* aChannel, nsILoadGroup* aLoadGroup) override;
   virtual void ResetToURI(nsIURI* aURI, nsILoadGroup* aLoadGroup,
-                          nsIPrincipal* aPrincipal) override;
+                          nsIPrincipal* aPrincipal,
+                          nsIPrincipal* aStoragePrincipal) override;
 
   virtual nsresult StartDocumentLoad(const char* aCommand, nsIChannel* aChannel,
                                      nsILoadGroup* aLoadGroup,
@@ -55,37 +65,29 @@ class nsHTMLDocument : public nsDocument, public nsIHTMLDocument {
                                      nsIStreamListener** aDocListener,
                                      bool aReset = true,
                                      nsIContentSink* aSink = nullptr) override;
-  virtual void StopDocumentLoad() override;
 
   virtual void BeginLoad() override;
   virtual void EndLoad() override;
 
+ protected:
+  virtual bool UseWidthDeviceWidthFallbackViewport() const override;
+
+ public:
   // nsIHTMLDocument
   virtual void SetCompatibilityMode(nsCompatibility aMode) override;
 
   virtual bool IsWriting() override { return mWriteLevel != uint32_t(0); }
 
-  virtual nsIContent* GetUnfocusedKeyEventTarget() override;
-
-  nsContentList* GetForms();
+  virtual Element* GetUnfocusedKeyEventTarget() override;
 
   nsContentList* GetExistingForms() const { return mForms; }
 
-  // nsIDOMDocument interface
-  using nsDocument::CreateElement;
-  using nsDocument::CreateElementNS;
-  NS_FORWARD_NSIDOMDOCUMENT(nsDocument::)
-
-  // And explicitly import the things from nsDocument that we just shadowed
-  using nsDocument::GetImplementation;
-  using nsDocument::GetLastStyleSheetSet;
-  using nsDocument::GetTitle;
-  using nsDocument::MozSetImageElement;
-  using nsDocument::SetTitle;
-
   mozilla::dom::HTMLAllCollection* All();
 
-  nsISupports* ResolveName(const nsAString& aName, nsWrapperCache** aCache);
+  // Returns whether an object was found for aName.
+  bool ResolveName(JSContext* aCx, const nsAString& aName,
+                   JS::MutableHandle<JS::Value> aRetval,
+                   mozilla::ErrorResult& aError);
 
   virtual void AddedForm() override;
   virtual void RemovedForm() override;
@@ -104,8 +106,6 @@ class nsHTMLDocument : public nsDocument, public nsIHTMLDocument {
 
   virtual EditingState GetEditingState() override { return mEditingState; }
 
-  virtual void DisableCookieAccess() override { mDisableCookieAccess = true; }
-
   class nsAutoEditingState {
    public:
     nsAutoEditingState(nsHTMLDocument* aDoc, EditingState aState)
@@ -120,21 +120,21 @@ class nsHTMLDocument : public nsDocument, public nsIHTMLDocument {
   };
   friend class nsAutoEditingState;
 
-  void EndUpdate(nsUpdateType aUpdateType) override;
+  void EndUpdate() override;
 
   virtual void SetMayStartLayout(bool aMayStartLayout) override;
 
   virtual nsresult SetEditingState(EditingState aState) override;
 
-  virtual nsresult Clone(mozilla::dom::NodeInfo* aNodeInfo, nsINode** aResult,
-                         bool aPreallocateChildren) const override;
+  virtual nsresult Clone(mozilla::dom::NodeInfo*,
+                         nsINode** aResult) const override;
 
   virtual void RemovedFromDocShell() override;
   using mozilla::dom::DocumentOrShadowRoot::GetElementById;
 
   virtual void DocAddSizeOfExcludingThis(
       nsWindowSizes& aWindowSizes) const override;
-  // DocAddSizeOfIncludingThis is inherited from nsIDocument.
+  // DocAddSizeOfIncludingThis is inherited from Document.
 
   virtual bool WillIgnoreCharsetOverride() override;
 
@@ -145,31 +145,24 @@ class nsHTMLDocument : public nsDocument, public nsIHTMLDocument {
   void SetDomain(const nsAString& aDomain, mozilla::ErrorResult& rv);
   bool IsRegistrableDomainSuffixOfOrEqualTo(const nsAString& aHostSuffixString,
                                             const nsACString& aOrigHost);
-  void GetCookie(nsAString& aCookie, mozilla::ErrorResult& rv);
-  void SetCookie(const nsAString& aCookie, mozilla::ErrorResult& rv);
   void NamedGetter(JSContext* cx, const nsAString& aName, bool& aFound,
                    JS::MutableHandle<JSObject*> aRetval,
-                   mozilla::ErrorResult& rv);
-  void GetSupportedNames(nsTArray<nsString>& aNames);
-  mozilla::dom::HTMLSharedElement* GetHead() {
-    return static_cast<mozilla::dom::HTMLSharedElement*>(GetHeadElement());
+                   mozilla::ErrorResult& rv) {
+    JS::Rooted<JS::Value> v(cx);
+    if ((aFound = ResolveName(cx, aName, &v, rv))) {
+      aRetval.set(v.toObjectOrNull());
+    }
   }
-  nsIHTMLCollection* Images();
-  nsIHTMLCollection* Embeds();
-  nsIHTMLCollection* Plugins();
-  nsIHTMLCollection* Links();
-  nsIHTMLCollection* Forms() { return nsHTMLDocument::GetForms(); }
-  nsIHTMLCollection* Scripts();
-  already_AddRefed<nsIDocument> Open(JSContext* cx, const nsAString& aType,
-                                     const nsAString& aReplace,
-                                     mozilla::ErrorResult& aError);
-  already_AddRefed<nsPIDOMWindowOuter> Open(
-      JSContext* cx, const nsAString& aURL, const nsAString& aName,
-      const nsAString& aFeatures, bool aReplace, mozilla::ErrorResult& rv);
+  void GetSupportedNames(nsTArray<nsString>& aNames);
+  Document* Open(const mozilla::dom::Optional<nsAString>& /* unused */,
+                 const nsAString& /* unused */, mozilla::ErrorResult& aError);
+  mozilla::dom::Nullable<mozilla::dom::WindowProxyHolder> Open(
+      const nsAString& aURL, const nsAString& aName, const nsAString& aFeatures,
+      bool aReplace, mozilla::ErrorResult& rv);
   void Close(mozilla::ErrorResult& rv);
-  void Write(JSContext* cx, const mozilla::dom::Sequence<nsString>& aText,
+  void Write(const mozilla::dom::Sequence<nsString>& aText,
              mozilla::ErrorResult& rv);
-  void Writeln(JSContext* cx, const mozilla::dom::Sequence<nsString>& aText,
+  void Writeln(const mozilla::dom::Sequence<nsString>& aText,
                mozilla::ErrorResult& rv);
   void GetDesignMode(nsAString& aDesignMode);
   void SetDesignMode(const nsAString& aDesignMode,
@@ -177,6 +170,7 @@ class nsHTMLDocument : public nsDocument, public nsIHTMLDocument {
   void SetDesignMode(const nsAString& aDesignMode,
                      const mozilla::Maybe<nsIPrincipal*>& aSubjectPrincipal,
                      mozilla::ErrorResult& rv);
+  MOZ_CAN_RUN_SCRIPT
   bool ExecCommand(const nsAString& aCommandID, bool aDoShowUI,
                    const nsAString& aValue, nsIPrincipal& aSubjectPrincipal,
                    mozilla::ErrorResult& rv);
@@ -188,6 +182,7 @@ class nsHTMLDocument : public nsDocument, public nsIHTMLDocument {
   bool QueryCommandState(const nsAString& aCommandID, mozilla::ErrorResult& rv);
   bool QueryCommandSupported(const nsAString& aCommandID,
                              mozilla::dom::CallerType aCallerType);
+  MOZ_CAN_RUN_SCRIPT
   void QueryCommandValue(const nsAString& aCommandID, nsAString& aValue,
                          mozilla::ErrorResult& rv);
   void GetFgColor(nsAString& aFgColor);
@@ -200,8 +195,6 @@ class nsHTMLDocument : public nsDocument, public nsIHTMLDocument {
   void SetAlinkColor(const nsAString& aAlinkColor);
   void GetBgColor(nsAString& aBgColor);
   void SetBgColor(const nsAString& aBgColor);
-  nsIHTMLCollection* Anchors();
-  nsIHTMLCollection* Applets();
   void Clear() const {
     // Deprecated
   }
@@ -209,17 +202,19 @@ class nsHTMLDocument : public nsDocument, public nsIHTMLDocument {
   void ReleaseEvents();
   // We're picking up GetLocation from Document
   already_AddRefed<mozilla::dom::Location> GetLocation() const {
-    return nsIDocument::GetLocation();
+    return Document::GetLocation();
   }
 
-  virtual nsHTMLDocument* AsHTMLDocument() override { return this; }
-
   static bool MatchFormControls(Element* aElement, int32_t aNamespaceID,
                                 nsAtom* aAtom, void* aData);
 
   void GetFormsAndFormControls(nsContentList** aFormList,
                                nsContentList** aFormControlList);
 
+  void UserInteractionForTesting();
+
+  void SetKeyPressEventModel(uint16_t aKeyPressEventModel);
+
  protected:
   ~nsHTMLDocument();
 
@@ -227,11 +222,6 @@ class nsHTMLDocument : public nsDocument, public nsIHTMLDocument {
 
   nsIContent* MatchId(nsIContent* aContent, const nsAString& aId);
 
-  static bool MatchLinks(mozilla::dom::Element* aElement, int32_t aNamespaceID,
-                         nsAtom* aAtom, void* aData);
-  static bool MatchAnchors(mozilla::dom::Element* aElement,
-                           int32_t aNamespaceID, nsAtom* aAtom, void* aData);
-
   static void DocumentWriteTerminationFunc(nsISupports* aRef);
 
   already_AddRefed<nsIURI> GetDomainURI();
@@ -240,19 +230,12 @@ class nsHTMLDocument : public nsDocument, public nsIHTMLDocument {
   already_AddRefed<nsIURI> RegistrableDomainSuffixOfInternal(
       const nsAString& aHostSuffixString, nsIURI* aOrigHost);
 
-  void WriteCommon(JSContext* cx, const nsAString& aText,
-                   bool aNewlineTerminate, mozilla::ErrorResult& aRv);
+  void WriteCommon(const nsAString& aText, bool aNewlineTerminate,
+                   mozilla::ErrorResult& aRv);
   // A version of WriteCommon used by WebIDL bindings
-  void WriteCommon(JSContext* cx, const mozilla::dom::Sequence<nsString>& aText,
+  void WriteCommon(const mozilla::dom::Sequence<nsString>& aText,
                    bool aNewlineTerminate, mozilla::ErrorResult& rv);
 
-  nsresult CreateAndAddWyciwygChannel(void);
-  nsresult RemoveWyciwygChannel(void);
-
-  // This should *ONLY* be used in GetCookie/SetCookie.
-  already_AddRefed<nsIChannel> CreateDummyChannelForCookies(
-      nsIURI* aCodebaseURI);
-
   /**
    * Like IsEditingOn(), but will flush as needed first.
    */
@@ -286,21 +269,11 @@ class nsHTMLDocument : public nsDocument, public nsIHTMLDocument {
   friend class ContentListHolder;
   ContentListHolder* mContentListHolder;
 
-  RefPtr<nsContentList> mImages;
-  RefPtr<nsEmptyContentList> mApplets;
-  RefPtr<nsContentList> mEmbeds;
-  RefPtr<nsContentList> mLinks;
-  RefPtr<nsContentList> mAnchors;
-  RefPtr<nsContentList> mScripts;
-  RefPtr<nsContentList> mForms;
-
   RefPtr<mozilla::dom::HTMLAllCollection> mAll;
 
   /** # of forms in the document, synchronously set */
   int32_t mNumForms;
 
-  static uint32_t gWyciwygSessionCnt;
-
   static void TryHintCharset(nsIContentViewer* aContentViewer,
                              int32_t& aCharsetSource,
                              NotNull<const Encoding*>& aEncoding);
@@ -313,12 +286,17 @@ class nsHTMLDocument : public nsDocument, public nsIHTMLDocument {
   void TryParentCharset(nsIDocShell* aDocShell, int32_t& charsetSource,
                         NotNull<const Encoding*>& aEncoding);
   void TryTLD(int32_t& aCharsetSource, NotNull<const Encoding*>& aCharset);
-  void TryFallback(int32_t& aCharsetSource,
-                   NotNull<const Encoding*>& aEncoding);
+  static void TryFallback(int32_t& aCharsetSource,
+                          NotNull<const Encoding*>& aEncoding);
 
-  // Override so we can munge the charset on our wyciwyg channel as needed.
-  virtual void SetDocumentCharacterSet(
-      NotNull<const Encoding*> aEncoding) override;
+  /**
+   * MaybeDispatchCheckKeyPressEventModelEvent() dispatches
+   * "CheckKeyPressEventModel" event to check whether we should dispatch
+   * keypress events in confluent model or split model.  This should be
+   * called only when mEditingState is changed to eDesignMode or
+   * eConentEditable at first time.
+   */
+  void MaybeDispatchCheckKeyPressEventModelEvent();
 
   // Tracks if we are currently processing any document.write calls (either
   // implicit or explicit). Note that if a write call writes out something which
@@ -335,30 +313,48 @@ class nsHTMLDocument : public nsDocument, public nsIHTMLDocument {
 
   bool mWarnedWidthHeight;
 
-  nsCOMPtr<nsIWyciwygChannel> mWyciwygChannel;
-
   /* Midas implementation */
-  nsresult GetMidasCommandManager(nsICommandManager** aCommandManager);
+  nsCommandManager* GetMidasCommandManager();
 
-  nsCOMPtr<nsICommandManager> mMidasCommandManager;
+  RefPtr<nsCommandManager> mMidasCommandManager;
 
   nsresult TurnEditingOff();
-  nsresult EditingStateChanged();
+  // MOZ_CAN_RUN_SCRIPT_BOUNDARY because this is called from all sorts
+  // of places, and I'm pretty sure the exact ExecCommand call it
+  // makes cannot actually run script.
+  MOZ_CAN_RUN_SCRIPT_BOUNDARY nsresult EditingStateChanged();
   void MaybeEditingStateChanged();
 
   uint32_t mContentEditableCount;
   EditingState mEditingState;
 
-  // When false, the .cookies property is completely disabled
-  bool mDisableCookieAccess;
-
   /**
    * Temporary flag that is set in EndUpdate() to ignore
    * MaybeEditingStateChanged() script runners from a nested scope.
    */
   bool mPendingMaybeEditingStateChanged;
+
+  // mHasBeenEditable is set to true when mEditingState is firstly set to
+  // eDesignMode or eContentEditable.
+  bool mHasBeenEditable;
+
+  /**
+   * Set to true once we know that we are loading plain text content.
+   */
+  bool mIsPlainText;
 };
 
+namespace mozilla {
+namespace dom {
+
+inline nsHTMLDocument* Document::AsHTMLDocument() {
+  MOZ_ASSERT(IsHTMLOrXHTML());
+  return static_cast<nsHTMLDocument*>(this);
+}
+
+}  // namespace dom
+}  // namespace mozilla
+
 #define NS_HTML_DOCUMENT_INTERFACE_TABLE_BEGIN(_class) \
   NS_DOCUMENT_INTERFACE_TABLE_BEGIN(_class)            \
   NS_INTERFACE_TABLE_ENTRY(_class, nsIHTMLDocument)
